import numpy as np
import sys
import cv2
import glob
import tensorflow as tf

# settings
input_model = "/home/dlwork02/pedet/saved_model"
output_model = "./models/output_quant.tflite"

# load validation set
image_folder = "/home/dlwork02/pedet/dai"
img_path = glob.glob(image_folder + "/*")
if len(img_path) == 0:
    print("image not found")
    sys.exit(1)

validation_data_set = []
for file_name in img_path:
    img = cv2.imread(file_name)  # BGR
    if img is None:
        print(f"Failed to load image: {file_name}")
        continue

    img = cv2.resize(img, (224, 224))
    ary = np.asarray(img, dtype=np.float32)
    #ary = ary / 127.5 - 1  # 正規化
    #ary = np.expand_dims(ary, axis=0)
    #validation_data_set.append(ary)
    ary = ary / 127.5 - 1  # 正規化
    ary_softmax = np.exp(ary) / np.sum(np.exp(ary), axis=1, keepdims=True)  # softmax化
    ary_softmax = np.expand_dims(ary_softmax, axis=0)  # バッチ次元を追加
    validation_data_set.append(ary_softmax)


# quantize
def representative_dataset_gen():
    for i in range(len(validation_data_set)):
        yield [validation_data_set[i]]

# converter
converter = tf.lite.TFLiteConverter.from_saved_model(input_model)
converter.optimizations = [tf.lite.Optimize.DEFAULT]
converter.representative_dataset = representative_dataset_gen
converter.target_spec.supported_ops = [tf.lite.OpsSet.TFLITE_BUILTINS_INT8]
converter.inference_input_type = tf.int8
converter.inference_output_type = tf.int8
#converter.inference_input_type = tf.float32
#converter.inference_output_type = tf.float32
tflite_quant_model = converter.convert()

with open(output_model, 'wb') as o_:
    o_.write(tflite_quant_model)
